function [f, sigDC, sigAC, fDC, fAC, hL, hD, Pdc] = ACsweep(opt, Nsweep)
%[f, sigDC, sigAC, fDC, fAC, hL, hD, Pdc] = ACsweep(opt, Nsweep)
%
% ACsweep.m produces AC response as hL for each DOF for each PD,
% hD for each DOF for port(REFL1I, ASDC, REFLDDM, OMCDDM, POXDDM).
%
% hL(frequendcy, PD number, length DOF)
% hD(frequendcy, port number, length DOF)
%
% Pdc is DC power for carrier and RF in each field defined as 'FN' in "setup.m".
%
% Pdc(field number)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%% Initial setting
  setup;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%% AC

  f = logspace(-1, 5, Nsweep)';
  [sigDC, sigAC, mechTF, fDC, fAC] = compute(opt, [], f);
  
  % ==== Optical transfer functions
  for pn = 1:length(PN)
    if isempty(findstr('IM', opt.probe(PN(pn)).name))
      if isempty(findstr('HD', opt.probe(PN(pn)).name))
        if isempty(findstr('DC', opt.probe(PN(pn)).name))
          if isempty(findstr('Q', opt.probe(PN(pn)).name))
            % single I
            DN = 1; ST = 1;
            dDC = 1;
          else
            % single Q
            DN = -1; ST = -1;
            dDC = 1;
          end
        else
          % DC readout
          DN = 0; ST = 1;
          dDC = 2; % DCreadout counts signal twice, probably this is an Optickle's bug.  
        end
      else
        % for Homodyne
        DN = 1; ST = 1;
        dDC = 2;
      end
    else
      % for DDM
      DN = 3; ST = 1;
      dDC = 1;  
    end

    for dn = 0 : ST : DN
      % hXX is from mirror position to sensor signal (W/m)
      hPR = squeeze(sigAC(PN(pn) + dn, nPR, :)) * qeff / dDC;
      hBS = squeeze(sigAC(PN(pn) + dn, nBS, :)) * qeff / dDC;
      hSR = squeeze(sigAC(PN(pn) + dn, nSR, :)) * qeff / dDC;
      hIX = squeeze(sigAC(PN(pn) + dn, nIX, :)) * qeff / dDC;
      hIY = squeeze(sigAC(PN(pn) + dn, nIY, :)) * qeff / dDC;
      hEX = squeeze(sigAC(PN(pn) + dn, nEX, :)) * qeff / dDC;
      hEY = squeeze(sigAC(PN(pn) + dn, nEY, :)) * qeff / dDC;

      hL(:, PN(pn) + dn, Lp) = (hEX + hEY);
      hL(:, PN(pn) + dn, Lm) = (hEX - hEY);
      hL(:, PN(pn) + dn, lp) = (hPR);
      hL(:, PN(pn) + dn, lm) = ((hEX-hIX) - (hEY-hIY)); % or hBS;
      %hL(:, PN(pn) + dn, lm) = hBS - hPR / sqrt(2) + hSR / sqrt(2); % hBS
      hL(:, PN(pn) + dn, lsr) = (hSR);
    end

    %%% hD: 
    if isempty(findstr('IM', opt.probe(PN(pn)).name))
      if isempty(findstr('HD', opt.probe(PN(pn)).name))
        hD(:, pn, :) = hL(:, PN(pn), :);
      else
        hD(:, pn, :) = hL(:, PN(pn), :) - hL(:, PN(pn)+1, :);
      end
    else % for DDM
      hD(:, pn, :) = ((hL(:, PN(pn), :) + hL(:, PN(pn)+2, :))) / 2;
    end
    
    % === used for flat shotnoise
%    Pdc(pn) = sum(abs(fDC(FN(pn), :).^2)) * qeff ;
    Pdc(pn) = sum(abs(fDC(FN(pn), :))).^2;

    if Pdc(pn) > PdcMAX(pn)
      fprintf('DC power at %s is %gW with T = %g attenuator, higher than maximum acceptable power %gW. Put more attenuator!\n', ...
                     char(ND(pn)),  Pdc(pn), (opt.optic(AttN(pn)).m(1,2))^2, PdcMAX(pn));
    else
      fprintf('DC power at %s is %gW with T = %g attenuator, less than maximum power %gW. OK...\n', ...
                     char(ND(pn)), Pdc(pn), (opt.optic(AttN(pn)).m(1,2))^2, PdcMAX(pn));
    end
  end
  
  % === Power estimation for PRC, arm cavity
    
  fDCp = permute(fDC, [2, 3, 1]);
  fprintf('\n\nPower budget on cavities.\n');
    
  n1 = getFieldAt(opt, nPR, 'bk');		% PR back input field
  for rf = 1 : length(nSBa) 
    [aPRb(rf),bPRb(rf)] = max(abs(fDCp(nSBa(rf), :, n1)));
    fprintf('Maximum peak %gW for %gMHz at the back surface of PRM.\n', aPRb(rf)^2, vFrf(nSBa(rf))/1e6);
  end
  fprintf('\n');

  n1 = getFieldOut(opt, nPR, 'fr');		% PR front output field
  for rf = 1 : length(nSBa) 
    [aPRf(rf),bPRf(rf)] = max(abs(fDCp(nSBa(rf), :, n1)));
    fprintf('Maximum peak %gW for %gMHz in front of PRM.\n', aPRf(rf)^2, vFrf(nSBa(rf))/1e6);
  end
  fprintf('\n');

  n1 = getFieldAt(opt, nIX, 'bk');		% IX back input field
  for rf = 1 : length(nSBa) 
    [aIXb(rf),bIXb(rf)] = max(abs(fDCp(nSBa(rf), :, n1)));
    fprintf('Maximum peak %gW for %gMHz at the back of ITMX.\n', aIXb(rf)^2, vFrf(nSBa(rf))/1e6);
  end
  
  fprintf('\n');
  n1 = getFieldOut(opt, nIX, 'fr');		% IX front output field
  for rf = 1 : length(nSBa) 
    [aIXf(rf),bIXf(rf)] = max(abs(fDCp(nSBa(rf), :, n1)));
    fprintf('Maximum peak %gW for %gMHz in front of ITMX.\n', aIXf(rf)^2, vFrf(nSBa(rf))/1e6);
  end
  fprintf('\n');

    n1 = getFieldAt(opt, nIY, 'bk');		% IY back input field
  for rf = 1 : length(nSBa) 
    [aIYb(rf),bIYb(rf)] = max(abs(fDCp(nSBa(rf), :, n1)));
    fprintf('Maximum peak %gW for %gMHz at the back of ITMY.\n', aIYb(rf)^2, vFrf(nSBa(rf))/1e6);
  end
  
  fprintf('\n');
  n1 = getFieldOut(opt, nIY, 'fr');		% IY front output field
  for rf = 1 : length(nSBa) 
    [aIYf(rf),bIYf(rf)] = max(abs(fDCp(nSBa(rf), :, n1)));
    fprintf('Maximum peak %gW for %gMHz in front of ITMX.\n', aIYf(rf)^2, vFrf(nSBa(rf))/1e6);
  end
  fprintf('\n');
  
  for rf = 1 : length(nSBa) 
    fprintf('Effective power recycling gain: %g for %gMHz.\n', (aPRf(rf)/aPRb(rf))^2, vFrf(nSBa(rf))/1e6);  
  end
  fprintf('\n');

  for rf = 1 : length(nSBa) 
    fprintf('Effective cavity gain: %g for %gMHz.\n', (aIXf(rf)/aIXb(rf))^2, vFrf(nSBa(rf))/1e6);  
  end
  fprintf('\n');
